#!/bin/python

import pgext
import pygame
import time

class Tester:
	def __init__(self,source_image,num=1):
		self.num = num
		self.source = pygame.image.load(source_image)
		self.tests = {}
		self.order = []
		self.results = {}
		self.times_start = {}
		self.times_result = {}
		self.types = ["modify-return","modify-inplace","make"]
		
	def addTests(self, tests):
		for t in tests:
			self.addTest(t)
		
	def addTest(self,test):
		source = self.source
		
		if len(test) == 4:
			name, typ, func, args = test
		else:
			name, typ, func, args, source = test
		
		if not typ in self.types:
			print "Unknown test type: %s"%typ
			return False
		self.tests[name] = ( typ, func, args, source )
		self.order.append(name)
		self.times_result[name] = []
		return True
		
	def saveImage(self,name,image):
		pygame.image.save(image, "out_%s.png"%name)
		
	def meterStart(self,name):
		self.times_start[name] = time.time()
		
	def meterEnd(self,name):
		self.times_result[name].append( (time.time()-self.times_start[name])*1000 )
		
	def doTests(self):
		for n in xrange(self.num):
			for test in self.order:
				self.doTest(test)
			
	def doTest(self,name):
		try:
			test = self.tests[name]
			source = test[3].copy()
			if test[0] == "modify-return":
				self.meterStart(name)
				image = test[1]( source, *test[2] )
				self.meterEnd(name)
				self.saveImage(name,image)
			elif test[0] == "modify-inplace":
				self.meterStart(name)
				test[1]( source, *test[2] )
				self.meterEnd(name)
				self.saveImage(name,source)
			elif test[0] == "make":
				self.meterStart(name)
				image = test[1]( *test[2] )
				self.meterEnd(name)
				self.saveImage(name,image)
			
			self.results[name] = 0
		except Exception,e:
			self.results[name] = e
			
	def printResults(self):
		print "Testing cycles: %s"%self.num
		count = 0
		for test in self.order:
			t = self.times_result[test]
			if self.results[test]:
				print "%s: Error (%s)"%( test, self.results[test] )
			else:
				avgtime = sum(self.times_result[test])/float(len(self.times_result[test]))
				count += avgtime
				print "%21s: %.1f ms"%( test, avgtime )
		print "%21s: %.1f ms"%( "Total time", count )

source_mask = pygame.image.load("source_mask.png")
source_rgb = pygame.image.load("source_rgb.png")

tester = Tester("source.png",10)
tester.addTests([
		("color.colorize", "modify-inplace", pgext.color.colorize, [300,40,-30,50] ),
		("color.greyscale", "modify-inplace", pgext.color.greyscale, []),
		("color.greyscale2", "modify-inplace", pgext.color.greyscale, [1]),
		("color.invert", "modify-inplace", pgext.color.invert, []),
		("color.hue", "modify-inplace", pgext.color.hue, [-50]),
		("color.saturation", "modify-inplace", pgext.color.saturation, [50,2]),
		("color.lightness", "modify-inplace", pgext.color.lightness, [-50]),
		("color.value", "modify-inplace", pgext.color.value, [-50]),
		("color.setColor", "modify-inplace", pgext.color.setColor, [(120,0,200)]),
		("color.alphaMask", "modify-inplace", pgext.color.alphaMask, [source_mask], source_rgb),
		
		("filters.shadow", "modify-return", pgext.filters.shadow, [(0,0,0), 5, 1, 0.9]),
		("filters.blur", "modify-inplace", pgext.filters.blur, [4]),
		("filters.hvBlur-h", "modify-inplace", pgext.filters.hvBlur, [6,0]),
		("filters.hvBlur-v", "modify-inplace", pgext.filters.hvBlur, [6,1]),
		("filters.noise", "modify-inplace", pgext.filters.noise, [55,2]),
		("filters.gradient1", "make", pgext.filters.gradient, [( 200, 200 ), ( 100,100,200 ), (100,200,100), 0, 1.0]),
		("filters.gradient2", "make", pgext.filters.gradient, [( 200, 200 ), ( 255,0,0 ), (0,0,255), 0, 0.3]),
		("filters.gradient3", "make", pgext.filters.gradient, [( 200,200 ), ( 100,0,0 ), (0,55,55), 1, 1.0]),
])
		
tester.doTests()
tester.printResults()



import math
import random
import pygame

def scratch(srf,row_size=3,offset=10,phase=1.0,rect=None):
    if offset <= 0:return False
    _scratch(False,srf,row_size,offset,phase,rect)

def vscratch(srf,col_size=3,offset=10,phase=1.0,rect=None):
    if offset <= 0:return False
    _scratch(True,srf,col_size,offset,phase,rect)

def _scratch(vertical,srf,linesize,offset,phase,rect):
    if rect:
        dx = rect[0]
        dy = rect[1]
        siz = (rect[2],rect[3])
    else:
        dx = 0
        dy = 0
        siz = srf.get_size()
    if vertical:
        sizlines = siz[0]
        h = siz[1]
        w = linesize
    else:
        sizlines = siz[1]
        w = siz[0]
        h = linesize
    for line in xrange(sizlines/linesize):
        if vertical:
            x = line*linesize
            ox = x
            y = round(((offset*2) * random.random())-offset)*phase
            oy = 0
        else:
            y = line*linesize
            oy = y
            x = round(((offset*2) * random.random())-offset)*phase
            ox = 0
        srf.blit(srf,(int(x)+dx,int(y)+dy),(ox+dx,oy+dy,w,h))


def wave(surface,amp,freq,wphase,vert=True):
    siz = surface.get_size()
    if vert:
        for line in xrange(siz[1]):
            num = math.sin( (wphase+line)*freq )
            y = line
            x = amp*num
            surface.blit(surface,(int(x),int(y)),(0,y,siz[0],1))
    else:
        for line in xrange(siz[0]):
            num = math.sin( (wphase+line)*freq )
            x = line
            y = amp*num
            surface.blit(surface,(int(x),int(y)),(x,0,1,siz[1]))

def shake(surface,offset,phase,rect=None):
    v = offset*phase
    rx = random.random()
    ry = random.random()
    if rx > 0.5:rx = 1
    if ry > 0.5:ry = 1
    if rx < 0.5:rx = -1
    if ry < 0.5:ry = -1
    x = int( (rx*v)/2 )
    y = int( (ry*v) )
    if rect:
        rec = ( rect[0]+x,rect[1]+y,rect[2],rect[3] )
        surface.blit(surface,(rect[0],rect[1]),rec)
    else:
        surface.blit(surface,(x,y))
    return surface

def roundedCorners(surface):
    w,h = surface.get_size()
    clr = None
    for p in ((0,0),(0,h-1),(w-1,0),(w-1,h-1)):
        clr = surface.get_at(p)
        surface.set_at(p,(0,0,0,0))
    a = (clr[3]/3)*2
    for p in ( (1,0),(0,1),  (1,h-1),(0,h-2), (w-2,0),(w-1,1), (w-2,h-1),(w-1,h-2) ):
        surface.set_at(p,(clr[0],clr[1],clr[2],a))
    for p in ( (1,1),(1,h-2), (w-2,1), (w-2,h-2) ):
        surface.set_at(p,(clr[0],clr[1],clr[2],clr[3]/2))

def partialBlit(surface,target_surface, rects, blend=0, offset=(0,0)):
    for r in rects:
        target_surface.blit( surface, ( int(r[0]+offset[0]), int(r[1]+offset[1]) ), r, blend )
